#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){
	
	// set background color to be white: 
	ofBackground(255, 255, 255);
	
	// this prevents banding on mac (vertical sync)
	ofSetVerticalSync(true);
	
	
	// load the image from the data folder
	myPicture.loadImage("test.png");
	
	// set the image type to be grayscale (1 byte per pixel)
	myPicture.setImageType(OF_IMAGE_GRAYSCALE);
	
	// allocate pixels and texture for this image. 
	width			= myPicture.width;
	height			= myPicture.height;
	pixels			= new unsigned char [width * height];
	texture.allocate(width, height, GL_LUMINANCE);
	
	// we need a place to blur into  (can't blur *in place*)
	pixelsAltered	= new unsigned char [width * height];
	
	
	
	
	
	// get a pointer to the image data:
	unsigned char * pixelsFromImage = myPicture.getPixels();
	
	// copy the pixels in...
	for (int i = 0; i < width; i++){
		for (int j = 0; j < height; j++){
			int pixIndex = j * width + i;
			pixels[pixIndex]  = pixelsFromImage[pixIndex];
		}
	}
	
	
	
	/*
	 
	 sharpen: 
	 
	 -1,-1,-1,
	 -1,9,-1,
	 -1,-1,-1 
	 weight 1
	 
	 gaussian: 
	 
	 1,2,1,
	 2,4,2,
	 1,2,1
	 weight: 16
	 
	 prewitt (edge): 
	 
	 1,1,1
	 0,0,0
	 -1,-1,-1
	 weight 0
	 
	 
	 */
	
	
	
	int kernel[9] = {	1,0,-1,
						1,0,-1,
						1,0,-1 };
	
	int kernelWeight = 0;
	
	
	// now, let's blur from pixels into pixelsBlurred:
	for (int i = 0; i < width; i++){
		for (int j = 0; j < height; j++){
			
			// deal with the edges !! :
			
			int pixIndex = j * width + i;
			
			int i_m_1 = MAX(0, i-1);
			int i_p_1 = MIN(width-1, i+1);
			int j_m_1 = MAX(0, j-1);
			int j_p_1 = MIN(height-1, j+1);
			
			int ne = (j_m_1 * width + i_m_1);
			int n_ = (j_m_1 * width + i    );
			int nw = (j_m_1 * width + i_p_1);
			int _e = (j     * width + i_m_1);
			int me = (j     * width + i    );
			int _w = (j     * width + i_p_1);
			int se = (j_p_1 * width + i_m_1);
			int s_ = (j_p_1 * width + i    );
			int sw = (j_p_1 * width + i_p_1);
			
			int newValue =	kernel[0] * pixels[ne] + kernel[1] * pixels[n_] + kernel[2] * pixels[nw] + 
							kernel[3] * pixels[_e] + kernel[4] * pixels[me] + kernel[5] * pixels[_w] + 
							kernel[6] * pixels[se] + kernel[7] * pixels[s_] + kernel[8] * pixels[sw];
			
			
			if (kernelWeight > 0){
				newValue /= kernelWeight;
			} else {
				
				if (newValue < 0) newValue = 0;
				if (newValue > 255) newValue = 255;
			}
			
			pixelsAltered[pixIndex]  = newValue;
		}
	}
	
	
	
	
	texture.loadData(pixelsAltered, width, height, GL_LUMINANCE);
	
		
}

//--------------------------------------------------------------
void testApp::update(){
	
	
		
	
}

//--------------------------------------------------------------
void testApp::draw(){
	

	// when we draw, the drawing color "tints" the drawing, so white is best for image
	// ie, white = no tint.
	ofSetColor(255, 255, 255);
	myPicture.draw(0,0);
	
	texture.draw(myPicture.width, 0);
	
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){

}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

